#include "audio.h"

#include <dirent.h>
#include <libgen.h>

#include <AL/alut.h>

#include <boost/foreach.hpp>

#include <iostream>

std::vector<std::string> Audio::filenames;
std::vector<ALuint> Audio::buffers;
std::vector<ALvoid*> Audio::audioData;

bool Audio::init (char* thispath) {
	if (! find(thispath)) {
		return false;
	}
	
	srand(time(0));
	
	std::random_shuffle(filenames.begin(), filenames.end());
	
	alutInit(0, 0);
	
	BOOST_FOREACH(std::string filename, filenames) {
		ALenum format;
		ALsizei length;
		ALfloat frequency;
		ALvoid* data;
		
		data = alutLoadMemoryFromFile (filename.c_str(), &format, &length, &frequency);
		
		ALuint bufferID;
		
		alGenBuffers(1, &bufferID);
		
		alBufferData(bufferID, format, data, length, frequency);
		
		buffers.push_back(bufferID);
		audioData.push_back(data);
		
		//buffers.push_back(alutCreateBufferFromFileImage(data, length));
	}
	
	atexit(Audio::cleanup);
	
	return true;
}

void Audio::cleanup () {
	alutExit();
}

bool Audio::find (char* thispath) {
	dirname(thispath);
	
	std::string audiodir = thispath;
	audiodir += "/audio";
	
	DIR* dirHandle = opendir(audiodir.c_str());

	if (! dirHandle) {
		return false;
	}

	struct dirent *dirEntry = readdir(dirHandle);
	
	while (dirEntry) {
		std::string filename = dirEntry->d_name;
		
		// if filename ends with ".wav"
		if (filename.find(".wav") == filename.length() - 4) {
			filenames.push_back(audiodir + "/" + filename);
		}
		
		dirEntry = readdir(dirHandle);
	}

	// Release the open directory
	closedir(dirHandle);
	
	return true;
}

