#include "audio.h"
#include "window.h"
#include "input.h"
#include "timer.h"
#include "player.h"
#include "emitter.h"
#include "floor.h"

#include "SDL_opengl.h"

#include <boost/foreach.hpp>
#include <iostream>
#include <vector>

ActivePlayer player;

std::vector<Emitter*> emitters;

Floor* theFloor;

void init () {
	// Try and initialise a window.
	if (! Window::init(800, 600, "Mini LD 9") ) {
		exit(1);
	}
	
	for (int i = 0; i < Audio::buffers.size(); i++) {
		int x = 3 * (i % 5) - 6;
		int z = 3 * (i / 5) - 6;
		emitters.push_back(new Emitter(i, x, z));
	}
	
	theFloor = new Floor();
}

void update () {
	
	if (Window::isFullscreen()) {
		SDL_ShowCursor(false);
	} else {
		SDL_ShowCursor(true);
	}
	
	player.update();
	
	BOOST_FOREACH(Emitter* emitter, emitters) {
		emitter->update();
	}
	
	theFloor->update();
}

void draw () {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	player.applyCamera();
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	theFloor->draw();
	
	player.draw();
	
	BOOST_FOREACH(Emitter* emitter, emitters) {
		emitter->draw();
	}
	
	SDL_GL_SwapBuffers();
}

int main (int argc, char** argv) {

	Audio::init(argv[0]);
	
	init();

	Timer timer;
	double previousTime = timer.getTime();
	double step = timer.getStep();
	double diff = 0.0;

	// Update/render loop.
	while (true) {
		// Check for input.
		Input::checkEvents();

		// Update the scene.
		update();

		// Draw the scene.
		draw();

		// Wait only if we have drawn quicker than the timer step.
		diff = timer.getTime() - previousTime;
		
		if (diff < step) {
			SDL_Delay( (int) ((step - diff) * 1000) );
		}
		previousTime = timer.getTime();
	}

	return 0;
}

