#include "player.h"
#include "input.h"
#include "window.h"
#include "drawing.h"
#include "globals.h"

#include "SDL_opengl.h"
#include <AL/al.h>

#include <math.h>

const float HEIGHT = 1.0f;
const float HALFHEIGHT = HEIGHT / 2;

void Player::draw () {
	glColor3f(0.8f,0.8f,0.8f);
	
	glPushMatrix();
	
	glTranslatef(x, 0, z);
	
	glRotatef(angle * 180 / 3.1415f, 0, 1, 0);
	
	glRotatef(leanForwards * 180 / 3.1415f, 1, 0, 0);
	
	glRotatef(leanSideways * 180 / 3.1415f, 0, 0, 1);
	
	float offset = sin(theFloor->frame * 0.75f) * 0.02f;
	
	glTranslatef(0, HALFHEIGHT + offset, 0);
	
	glPushMatrix();
	
	glScalef(HEIGHT / 5, HALFHEIGHT + offset, HEIGHT / 5);
	
	glCallList(Drawing::sphereDisplayList);
	
	glPopMatrix();
	
	/*glTranslatef(0, HALFHEIGHT, 0);
	
	glRotatef(-leanSideways * 90 / 3.1415f, 0, 0, 1);
	
	glRotatef(-leanForwards * 180 / 3.1415f, 1, 0, 0);
	
	glTranslatef(0, sin(theFloor->frame * 0.75f) * 0.02f, 0);
	
	glScalef(0.75f, 0.75f, 0.75f);
	
	glPushMatrix();
	
	glScalef(HALFHEIGHT, HALFHEIGHT / 4, HALFHEIGHT);
	
	glCallList(Drawing::sphereDisplayList);
	
	glPopMatrix();
	
	glColor3f(0.2f,0.2f,0.2f);
	
	glPushMatrix();
	
	glTranslatef(-HEIGHT / 3.5, HEIGHT / 9, 0);
	
	glScalef(HALFHEIGHT / 5, HALFHEIGHT / 5, HALFHEIGHT / 16);
	
	glCallList(Drawing::sphereDisplayList);
	
	glPopMatrix();
	
	glPushMatrix();
	
	glTranslatef(HEIGHT / 3.5, HEIGHT / 9, 0);
	
	glScalef(HALFHEIGHT / 5, HALFHEIGHT / 5, HALFHEIGHT / 16);
	
	glCallList(Drawing::sphereDisplayList);
	
	glPopMatrix();*/
	
	glPopMatrix();
}

void ActivePlayer::update () {
    ALfloat	orientation[] = {sin(angle), 0.0, cos(angle), 0.0,1.0,0.0};
    alListenerfv(AL_ORIENTATION, orientation);
    
    float angleChange = 0;
    
    if (Input::isPressed(SDLK_LEFT)) {
    	angleChange += 0.1f;
    }
    if (Input::isPressed(SDLK_RIGHT)) {
    	angleChange -= 0.1f;
    }
    
    if (angleChange < -0.01f) {
    	leanSideways = 0.8f * (0.1f + leanSideways);
    } else if (angleChange > 0.01f) {
    	leanSideways = 0.8f * (-0.1f + leanSideways);
    } else {
    	leanSideways *= 0.8f;
    }
    
    angle += angleChange;
    
    float move = 0;
    
    if (Input::isPressed(SDLK_UP)) {
    	move += 0.4f;
    }
    if (Input::isPressed(SDLK_DOWN)) {
    	move -= 0.4f;
    }
    
    if (move > 0.1f) {
    	leanForwards = 0.8f * (0.2f + leanForwards);
    } else if (move < -0.1f) {
    	leanForwards = 0.8f * (-0.15f + leanForwards);
    } else {
    	leanForwards *= 0.8f;
    }
    
    vx = move * sin(angle);
    vz = move * cos(angle);
    
    x += vx;
    z += vz;
    
	alListener3f(AL_POSITION, x, HEIGHT, z);
    alListener3f(AL_VELOCITY, vx, 0, vz);
}

void ActivePlayer::applyCamera () {
	int width = Window::getWidth();
	int height = Window::getHeight();
	
	// Make it scale when you resize window or go fullscreen
	height = (int) (height * 640.0f / width);
	width = 640;
	
	// Set up the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 1, 400);
	
	const float distance = 7;
	
	gluLookAt(x - sin(angle) * distance, 2*HEIGHT, z - cos(angle) * distance, x, HEIGHT, z, 0, 1, 0);
	
	/*glTranslatef(x, 2*HEIGHT, z);
	
	glRotatef(angle, 0, 1, 0);
	
	glTranslatef(0, 0, HEIGHT);*/
	
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	
	// Lighting
	
	glShadeModel (GL_SMOOTH);
	
	glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE ) ;
	glEnable ( GL_COLOR_MATERIAL ) ;
	
	GLfloat global_ambient[] = {0.5f, 0.5f, 0.5f, 1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
	
	GLfloat ambient[] = {0.0f, 0.0f, 0.0f, 1.0f};
	GLfloat diffuse[] = {0.8f, 0.8f, 0.8f, 1.0f};
	GLfloat specular[] = {0.6f, 0.6f, 0.6f, 1.0f};
	GLfloat emission[] = {0.1f, 0.1f, 0.1f, 1.0f};
	
	GLfloat light_position[] = { 0.0, 4.0, 0.0, 1.0 };
	
	glMaterialfv ( GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv ( GL_FRONT_AND_BACK, GL_EMISSION, emission);

	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	glEnable(GL_NORMALIZE);
	
	// Make lines look better
	
	/*glEnable( GL_LINE_SMOOTH );
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	
	glLineWidth (1.5);*/
	
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

